/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.jmx.artifact;

import java.net.URL;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Logger;

import javax.management.MBeanAttributeInfo;

import org.ow2.petals.admin.api.artifact.ArtifactState.State;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.lifecycle.ComponentLifecycle;
import org.ow2.petals.admin.api.conf.PetalsAdminConfiguration;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactStartedException;
import org.ow2.petals.admin.api.exception.ArtifactStoppedException;
import org.ow2.petals.admin.api.exception.ArtifactUnsupportedOperationException;
import org.ow2.petals.admin.api.exception.ArtifactUrlRewriterException;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.ConfigurationInstallerComponentClient;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentUnsupportedOperationException;
import org.ow2.petals.jmx.api.api.exception.ConfigurationInstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ConfigurationInstallerComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentErrorException;

/**
 * This class represents a remote JBI Component.
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class JMXComponentLifecycle extends AbstractLifecycle implements ComponentLifecycle {

    private final Logger LOG = Logger.getLogger(JMXComponentLifecycle.class.getName());

    private final Component component;

    protected JMXComponentLifecycle(final Component c, final PetalsAdminConfiguration configuration) {
        super(configuration);
        this.component = c;
    }

    public Component getComponent() {
        return this.component;
    }

    /**
     * {@inheritDoc}
     * <p>
     * Special note for component deployment:
     * <ul>
     * <li>if the component installer is already loaded, it will be unloaded
     * before to reload a new one.</li>
     * <li>if the component is already installed and in state 'SHUTDOWN', it
     * will be uninstalled before to unload the component installer and reload a
     * new one,</li>
     * <li>otherwise an error is thrown.</li>
     * </ul>
     * </p>
     */
    @Override
    public void deploy(final URL artifactUrl) throws ArtifactDeployedException,
            ArtifactAdministrationException {
        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final String componentName = this.getComponent().getName();
            final InstallationServiceClient installSvcClt = client.getInstallationServiceClient();
            InstallerComponentClient installer;
            try {
                installer = installSvcClt.loadInstaller(componentName);
                // A previous installer already exist:
                // - if the component is already installed and in state
                // 'SHUTDOWN', we uninstall it before to unload the component
                // installer and reload a new one,
                // - if the component is not already installed, we unload the
                // component installer and reload a new one,
                // - otherwise an error is thrown
                try {
                    final ComponentClient componentCltSvc = client
                            .getComponentClient(componentName);
                    final ComponentClient.State state = componentCltSvc.getCurrentState();
                    if (state == ComponentClient.State.SHUTDOWN) {
                        // The component is installed and in state 'SHUTDOWN'
                        LOG.warning("The component is in state '"
                                + ComponentClient.State.SHUTDOWN.name()
                                + "', it has been automatically uninstalled and unloaded.");
                        installer.uninstall();
                    } else {
                        throw new ArtifactDeployedException(componentName, state.toString());
                    }
                } catch (final ComponentDoesNotExistException e) {
                    // The component is not already installed
                    LOG.warning("The component installer is already loaded and only loaded, it has been automatically unloaded.");
                }

                if (installSvcClt.unloadInstaller(componentName)) {
                    installer = installSvcClt.loadNewInstaller(this.getConfiguration().getArtifactUrlRewriter()
                            .rewrite(artifactUrl));
                } else {
                    throw new ArtifactAdministrationException(
                            "Impossible to unload the previously existing component installer");
                }
            } catch (final InstallerComponentDoesNotExistException e) {
                installer = installSvcClt.loadNewInstaller(this.getConfiguration().getArtifactUrlRewriter()
                        .rewrite(artifactUrl));
            }

            // Set the JBI parameters if they exist 
            final Properties parameters = this.getComponent().getParameters();
            if (parameters != null && parameters.size() > 0) {
                try {
                    final ConfigurationInstallerComponentClient installerConfigurationMBean = installer
                            .getConfigurationInstallerClient();

                    if (installerConfigurationMBean != null) {
                        for (String parameterName : parameters.stringPropertyNames()) {
                            setAttribute(installerConfigurationMBean, parameterName,
                                    parameters.getProperty(parameterName));
                        }
                    } else {
                        throw new ArtifactAdministrationException(
                                "Impossible to set parameters for this component (no installer configuration MBean)");
                    }
                } catch (final ConfigurationInstallerComponentDoesNotExistException e) {
                    throw new ArtifactAdministrationException(
                            "Impossible to set parameters for this component (no installer configuration MBean)", e);
                }
            }

            // Install the component
            installer.install();

        } catch (final ConnectionErrorException cee) {
            throw new ArtifactAdministrationException(cee);
        } catch (final InstallationServiceErrorException isee) {
            throw new ArtifactAdministrationException(isee);
        } catch (final InstallationServiceDoesNotExistException isdnee) {
            throw new ArtifactAdministrationException(isdnee);
        } catch (final ComponentErrorException e) {
            throw new ArtifactAdministrationException(e);
        } catch (final InstallerComponentErrorException e) {
            throw new ArtifactAdministrationException(e);
        } catch (final ArtifactUrlRewriterException e) {
            throw new ArtifactAdministrationException(e);
        } catch (final ConfigurationInstallerComponentErrorException e) {
            throw new ArtifactAdministrationException(e);
        }
    }

    private static final void setAttribute(
            ConfigurationInstallerComponentClient installerConfigurationMBean,
            String parameterName, String parameterStringValue)
            throws ArtifactAdministrationException {
        assert parameterStringValue != null;

        try {
            Set<MBeanAttributeInfo> configurationAttributes = installerConfigurationMBean
                    .getAttributes().keySet();

            Object parameterValue = null;
            boolean found = false;
            Iterator<MBeanAttributeInfo> configurationAttributesIterator = configurationAttributes
                    .iterator();
            while (configurationAttributesIterator.hasNext() && !found) {
                MBeanAttributeInfo attributeInfo = configurationAttributesIterator.next();
                if (attributeInfo.getName().equals(parameterName)) {

                    try {
                        final String attributeType = attributeInfo.getType();
                        if (String.class.getName().equals(attributeType)) {
                            parameterValue = parameterStringValue;
                        } else if ("int".equals(attributeType) || Integer.class.getName().equals(attributeType)) {
                            parameterValue = Integer.valueOf(parameterStringValue);
                        } else if ("long".equals(attributeType) || Long.class.getName().equals(attributeType)) {
                            parameterValue = Long.valueOf(parameterStringValue);
                        } else if ("short".equals(attributeType) || Short.class.getName().equals(attributeType)) {
                            parameterValue = Short.valueOf(parameterStringValue);
                        } else if ("boolean".equals(attributeType) || Boolean.class.getName().equals(attributeType)) {
                            parameterValue = Boolean.valueOf(parameterStringValue);
                        } else {
                            throw new ArtifactAdministrationException(String.format(
                                    "The type '%s' of parameter '%s' is not supported.",
                                    attributeType, parameterName));
                        }
                        found = true;
                    } catch (final NumberFormatException nfe) {
                        String errorMessage = String.format(
                                "The value of the parameter '%s' is incorrect", parameterName);
                        throw new ArtifactAdministrationException(errorMessage, nfe);
                    }
                }
            }

            if (!found) {
                String errorMessage = String.format(
                        "The parameter '%s' does not exist or is not settable for this component",
                        parameterName);
                throw new ArtifactAdministrationException(errorMessage);
            }

            installerConfigurationMBean.setAttribute(parameterName, parameterValue);
        } catch (AttributeErrorException aee) {
            throw new ArtifactAdministrationException(aee);
        }
    }

    /**
     * {@inheritDoc}
     * <p>
     * Special note for component start:
     * <ul>
     * <li>if the component is in state 'STARTED', the error
     * {@link ArtifactStartedException} is raised,</li>
     * <li>if the component is in state 'STOPPED', it will be started,</li>
     * <li>if the component is in state 'SHUTDOWN', it will be started,</li>
     * <li>if the component is not installed but component installer is loaded,
     * the error {@link ArtifactNotDeployedException} is raised,</li>
     * <li>if the installer is not loaded, the error
     * {@link ArtifactNotFoundException} is raised.</li>
     * </ul>
     * </p>
     */
    @Override
    public void start() throws ArtifactStartedException, ArtifactNotDeployedException,
            ArtifactNotFoundException, ArtifactAdministrationException {
        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final String componentName = this.getComponent().getName();

            try {
                final ComponentClient componentCltSvc = client.getComponentClient(componentName);
                final ComponentClient.State state = componentCltSvc.getCurrentState();
                if (state == ComponentClient.State.STARTED) {
                    // The component is installed and its state is 'STARTED'
                    throw new ArtifactStartedException(componentName, this.getComponent().getType());
                }

                componentCltSvc.start();

            } catch (final ComponentDoesNotExistException e) {
                try {
                    // The component is not already installed, we check if the
                    // component installer is loaded
                    client.getInstallationServiceClient().loadInstaller(componentName);
                    // Here, we have checked that the component installer was
                    // loaded
                    throw new ArtifactNotDeployedException(this.getComponent().getType(),
                            componentName);
                } catch (final InstallerComponentDoesNotExistException e1) {
                    // The component installer is not loaded
                    throw new ArtifactNotFoundException(this.getComponent().getType(), componentName, this
                            .getComponent().getVersion());
                }
            }
        } catch (ConnectionErrorException cee) {
            throw new ArtifactAdministrationException(cee);
        } catch (InstallationServiceErrorException isee) {
            throw new ArtifactAdministrationException(isee);
        } catch (InstallationServiceDoesNotExistException isdnee) {
            throw new ArtifactAdministrationException(isdnee);
        } catch (final ComponentErrorException e) {
            throw new ArtifactAdministrationException(e);
        }
    }

    /**
     * {@inheritDoc}
     * <p>
     * Special note for component stop:
     * <ul>
     * <li>if the component is in state 'STARTED', it will be stopped,</li>
     * <li>if the component is in state 'STOPPED', the error
     * {@link ArtifactStoppedException} is raised,</li>
     * <li>if the component is in state 'SHUTDOWN', the error
     * {@link ArtifactStoppedException} is raised,</li>
     * <li>if the component is not installed but component installer is loaded,
     * the error {@link ArtifactNotDeployedException} is raised,</li>
     * <li>if the installer is not loaded, the error
     * {@link ArtifactNotFoundException} is raised.</li>
     * </ul>
     * </p>
     */
    @Override
    public void stop() throws ArtifactStoppedException, ArtifactNotDeployedException,
            ArtifactNotFoundException, ArtifactAdministrationException {
        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final String componentName = this.getComponent().getName();
            try {
                final ComponentClient componentCltSvc = client
                        .getComponentClient(componentName);
                final ComponentClient.State state = componentCltSvc.getCurrentState();
                if (state == ComponentClient.State.STOPPED || state == ComponentClient.State.SHUTDOWN ) {
                    // The component is installed and its state is 'STOPPED' or 'SHUTDOWN'
                    throw new ArtifactStoppedException(componentName, this.getComponent()
                            .getType());
                }
                
                componentCltSvc.stop();
                
            } catch (final ComponentDoesNotExistException e) {
                try {
                    // The component is not already installed, we check if the component installer is loaded
                    client.getInstallationServiceClient().loadInstaller(componentName);
                    // Here, we have checked that the component installer was loaded
                    throw new ArtifactNotDeployedException(this.getComponent().getType(),
                            componentName);
                } catch (final InstallerComponentDoesNotExistException e1) {
                    // The component installer is not loaded
                    throw new ArtifactNotFoundException(this.getComponent().getType(), componentName, this
                            .getComponent().getVersion());
                }
            }
            
        } catch (final ConnectionErrorException cee) {
            throw new ArtifactAdministrationException(cee);
        } catch (final InstallationServiceErrorException isee) {
            throw new ArtifactAdministrationException(isee);
        } catch (final InstallationServiceDoesNotExistException isdnee) {
            throw new ArtifactAdministrationException(isdnee);
        } catch (final ComponentErrorException e) {
            throw new ArtifactAdministrationException(e);
        }
    }

    /**
     * {@inheritDoc}
     * <p>
     * Special note for component undeployment:
     * <ul>
     * <li>if the component is in state 'STARTED', the error
     * {@link ArtifactStartedException} is raised,</li>
     * <li>if the component is in state 'STOPPED', the component will be
     * shutdown, uninstalled and the component installer will be unloaded,</li>
     * <li>if the component is in state 'SHUTDOWN', the component will be
     * uninstalled and the component installer will be unloaded,</li>
     * <li>if the component is not installed but component installer is loaded,
     * the component installer will be unloaded,</li>
     * <li>if the installer is not loaded, the error
     * {@link ArtifactNotFoundException} is raised.</li>
     * </ul>
     * </p>
     */
    @Override
    public void undeploy() throws ArtifactStartedException, ArtifactNotFoundException,
            ArtifactAdministrationException {
        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final String componentName = this.getComponent().getName();
            final InstallationServiceClient installSvcClt = client.getInstallationServiceClient();

            try {
                final ComponentClient componentCltSvc = client.getComponentClient(componentName);
                final ComponentClient.State state = componentCltSvc.getCurrentState();
                final InstallerComponentClient installer = installSvcClt
                        .loadInstaller(componentName);
                if (state == ComponentClient.State.STARTED) {
                    throw new ArtifactStartedException(componentName, this.getComponent().getType());
                } else if (state == ComponentClient.State.STOPPED) {
                    componentCltSvc.shutdown();
                    installer.uninstall();
                } else if (state == ComponentClient.State.SHUTDOWN) {

                    installer.uninstall();
                } else {
                    throw new ArtifactAdministrationException("The artifact '" + componentName
                            + "' is in an unknown state '" + state + "'.");
                }
            } catch (final ComponentDoesNotExistException e) {
                // The component is not installed
                try {
                    installSvcClt.loadInstaller(componentName);
                    LOG.warning("The component is not installed but loaded, it will be automatically unloaded.");
                } catch (final InstallerComponentDoesNotExistException e1) {
                    // The component installer is not loaded
                    throw new ArtifactNotFoundException(componentName, this.getComponent().getType(), this
                            .getComponent().getVersion());
                }
            }

            // Here, the component installer should be unload
            final boolean isSuccessful = installSvcClt.unloadInstaller(componentName);
            if (!isSuccessful) {
                final String errorMessage = String.format("Failed to shutdown the component '%s'",
                        componentName);
                throw new ArtifactAdministrationException(errorMessage.toString());
            }

        } catch (final ConnectionErrorException cee) {
            throw new ArtifactAdministrationException(cee);
        } catch (final InstallationServiceErrorException isee) {
            throw new ArtifactAdministrationException(isee);
        } catch (final InstallationServiceDoesNotExistException isdnee) {
            throw new ArtifactAdministrationException(isdnee);
        } catch (final InstallerComponentDoesNotExistException icdnee) {
            throw new ArtifactAdministrationException(icdnee);
        } catch (final ComponentErrorException e) {
            throw new ArtifactAdministrationException(e);
        } catch (final InstallerComponentErrorException e) {
            throw new ArtifactAdministrationException(e);
        }
    }

    @Override
    public void updateState() throws ArtifactAdministrationException {
        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final String componentName = this.getComponent().getName();
            final ComponentClient.State state = client.getComponentClient(componentName)
                    .getCurrentState();
            this.getComponent().setState(State.fromStringValue(state.toString()));
        } catch (final ConnectionErrorException cee) {
            throw new ArtifactAdministrationException(cee);
        } catch (final ComponentErrorException cee) {
            throw new ArtifactAdministrationException(cee);
        } catch (final ComponentDoesNotExistException cdnee) {
            throw new ArtifactAdministrationException(cdnee);
        }
    }

    /**
     * {@inheritDoc}
     * <p>
     * Special note for component reload configuration:
     * <ul>
     * <li>if the component is not installed but component installer is loaded, the error
     * {@link ArtifactNotFoundException} is raised,</li>
     * <li>if the installer is not loaded, the error {@link ArtifactNotFoundException} is raised.</li>
     * </ul>
     * </p>
     */
    @Override
    public void reloadConfiguration() throws ArtifactNotFoundException, ArtifactUnsupportedOperationException,
            ArtifactAdministrationException {
        try {
            final JMXClient client = JMXClientConnection.getJMXClient();
            final String componentName = this.getComponent().getName();
            try {
                final ComponentClient componentCltSvc = client.getComponentClient(componentName);

                componentCltSvc.reloadConfiguration();

            } catch (final ComponentDoesNotExistException e) {
                throw new ArtifactNotFoundException(this.getComponent().getType(), componentName, this.getComponent()
                        .getVersion());
            } catch (final ComponentUnsupportedOperationException e) {
                throw new ArtifactUnsupportedOperationException();
            }

        } catch (final ConnectionErrorException cee) {
            throw new ArtifactAdministrationException(cee);
        } catch (final ComponentErrorException e) {
            throw new ArtifactAdministrationException(e);
        }
    }
}
