/**
 * Copyright (c) 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.jmx;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.util.List;

import org.junit.Test;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Domain;
import org.ow2.petals.admin.topology.Subdomain;
import org.ow2.petals.jmx.api.mock.PetalsJmxApiFactoryMock;
import org.ow2.petals.jmx.api.mock.TopologyServiceClientMock;

/**
 * Unit tests of {@link JMXContainerAdministration}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class JMXContainerAdministrationTest extends JMXAbstractAdministrationTest {

    /**
     * <p>
     * Check the
     * {@link JMXContainerAdministration#connect(String, int, String, String)}
     * when the JMX connection is succeeded.
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     */
    @Test
    public void testConnect_00() throws DuplicatedServiceException, MissingServiceException,
            ContainerAdministrationException {
        final ContainerAdministration containerAdm = PetalsAdministrationFactory.newInstance()
                .newContainerAdministration();
        containerAdm.connect("localhost", 7700, "petals", "petals");
    }

    /**
     * <p>
     * Check the
     * {@link JMXContainerAdministration#connect(String, int, String, String)}
     * when the JMX connection failed.
     * </p>
     * <p>
     * Expected results: The exception {@link ContainerAdministrationException}
     * occurs.
     * </p>
     */
    @Test(expected = ContainerAdministrationException.class)
    public void testConnect_01() throws DuplicatedServiceException, MissingServiceException,
            ContainerAdministrationException {
        final ContainerAdministration containerAdm = PetalsAdministrationFactory.newInstance()
                .newContainerAdministration();
        containerAdm.connect(PetalsJmxApiFactoryMock.HOST_ERROR_CREATING_JMX_CLIENT, 7700,
                "petals", "petals");
    }

    /**
     * <p>
     * Check the {@link JMXContainerAdministration#disconnect()} when:
     * <ul>
     * <li>the JMX connection is succeeded,</li>
     * <li>the JMX disconnection is succeeded,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs.
     * </p>
     */
    @Test
    public void testDisconnect_00() throws DuplicatedServiceException, MissingServiceException,
            ContainerAdministrationException {
        final ContainerAdministration containerAdm = PetalsAdministrationFactory.newInstance()
                .newContainerAdministration();
        containerAdm.connect("localhost", 7700, "petals", "petals");
        containerAdm.disconnect();
    }

    /**
     * <p>
     * Check the {@link JMXContainerAdministration#disconnect()} when no JMX
     * connection is established.
     * </p>
     * <p>
     * Expected results: The exception {@link ContainerAdministrationException}
     * occurs.
     * </p>
     */
    @Test(expected = ContainerAdministrationException.class)
    public void testDisconnect_01() throws DuplicatedServiceException, MissingServiceException,
            ContainerAdministrationException {
        final ContainerAdministration containerAdm = PetalsAdministrationFactory.newInstance()
                .newContainerAdministration();
        containerAdm.disconnect();
    }

    /**
     * Check the {@link JMXContainerAdministration#getTopology(String, String)} against container filtering where:
     * <ul>
     * <li>the topology is composed of:
     * <ul>
     * <li>a sub-domain 'subdomain-0' containing the containers 'container-0-0' and 'container-0-1',</li>
     * <li>a sub-domain 'subdomain-1' containing the containers 'container-1-0' and 'container-1-1',</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>when filtering with the regexp 'container-.*':
     * <ul>
     * <li>no error,</li>
     * <li>all containers are returned, each in its right sub-domain.</li>
     * </ul>
     * </li>
     * <li>when filtering with the regexp 'container-[01]-1':
     * <ul>
     * <li>no error,</li>
     * <li>only the containers 'container-0-1' and 'container-1-1' are returned, each in its right sub-domain.</li>
     * </ul>
     * </li>
     * <li>when filtering with the regexp 'container-0-1':
     * <ul>
     * <li>no error,</li>
     * <li>only the container 'container-0-1' is returned in its right sub-domain.</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     */
    @Test
    public void testGetTopology() throws ContainerAdministrationException, DuplicatedServiceException,
            MissingServiceException {
        final ContainerAdministration containerAdm = PetalsAdministrationFactory.newInstance()
                .newContainerAdministration();
        containerAdm.connect("localhost", 7700, "petals", "petals");

        // Check against container name regexp: 'container-.*'
        {
            final Domain domain = containerAdm.getTopology("container-.*",
                    TopologyServiceClientMock.TOPO_4_CONTAINERS_PASSPHRASE);
            assertEquals(TopologyServiceClientMock.DOMAIN_NAME, domain.getName());
            final List<Subdomain> subdomains = domain.getSubdomains();
            assertEquals(2, subdomains.size());
            Subdomain subdomain0 = null;
            Subdomain subdomain1 = null;
            for (final Subdomain subdomain : subdomains) {
                if (TopologyServiceClientMock.SUBDOMAIN_0_NAME.equals(subdomain.getName())) {
                    subdomain0 = subdomain;
                } else if (TopologyServiceClientMock.SUBDOMAIN_1_NAME.equals(subdomain.getName())) {
                    subdomain1 = subdomain;
                } else {
                    fail("Unexpected sub-domain found: " + subdomain.getName());
                }
            }
            assertNotNull("Sub-domain #0 is missing", subdomain0);
            assertNotNull("Sub-domain #1 is missing", subdomain1);

            {
                final List<Container> containers = subdomain0.getContainers();
                assertEquals(2, containers.size());
                Container container00 = null;
                Container container01 = null;
                for (final Container container : containers) {
                    if (TopologyServiceClientMock.CONTAINER_00_NAME.equals(container.getContainerName())) {
                        container00 = container;
                    } else if (TopologyServiceClientMock.CONTAINER_01_NAME.equals(container.getContainerName())) {
                        container01 = container;
                    } else {
                        fail("Unexpected container found: " + container.getContainerName());
                    }
                }
                assertNotNull("Container #00 is missing", container00);
                assertNotNull("Container #01 is missing", container01);
            }

            {
                final List<Container> containers = subdomain1.getContainers();
                assertEquals(2, containers.size());
                Container container10 = null;
                Container container11 = null;
                for (final Container container : containers) {
                    if (TopologyServiceClientMock.CONTAINER_10_NAME.equals(container.getContainerName())) {
                        container10 = container;
                    } else if (TopologyServiceClientMock.CONTAINER_11_NAME.equals(container.getContainerName())) {
                        container11 = container;
                    } else {
                        fail("Unexpected container found: " + container.getContainerName());
                    }
                }
                assertNotNull("Container #10 is missing", container10);
                assertNotNull("Container #11 is missing", container11);
            }
        }

        // Check against container name regexp: 'container-[01]-1'
        {
            final Domain domain = containerAdm.getTopology("container-[01]-1",
                    TopologyServiceClientMock.TOPO_4_CONTAINERS_PASSPHRASE);
            assertEquals(TopologyServiceClientMock.DOMAIN_NAME, domain.getName());
            final List<Subdomain> subdomains = domain.getSubdomains();
            assertEquals(2, subdomains.size());
            Subdomain subdomain0 = null;
            Subdomain subdomain1 = null;
            for (final Subdomain subdomain : subdomains) {
                if (TopologyServiceClientMock.SUBDOMAIN_0_NAME.equals(subdomain.getName())) {
                    subdomain0 = subdomain;
                } else if (TopologyServiceClientMock.SUBDOMAIN_1_NAME.equals(subdomain.getName())) {
                    subdomain1 = subdomain;
                } else {
                    fail("Unexpected sub-domain found: " + subdomain.getName());
                }
            }
            assertNotNull("Sub-domain #0 is missing", subdomain0);
            assertNotNull("Sub-domain #1 is missing", subdomain1);

            {
                final List<Container> containers = subdomain0.getContainers();
                assertEquals(1, containers.size());
                assertEquals("Container #01 is missing", TopologyServiceClientMock.CONTAINER_01_NAME, containers.get(0)
                        .getContainerName());
            }

            {
                final List<Container> containers = subdomain1.getContainers();
                assertEquals(1, containers.size());
                assertEquals("Container #11 is missing", TopologyServiceClientMock.CONTAINER_11_NAME, containers.get(0)
                        .getContainerName());
            }
        }

        // Check against container name regexp: 'container-0-1'
        {
            final Domain domain = containerAdm.getTopology("container-0-1",
                    TopologyServiceClientMock.TOPO_4_CONTAINERS_PASSPHRASE);
            assertEquals(TopologyServiceClientMock.DOMAIN_NAME, domain.getName());
            final List<Subdomain> subdomains = domain.getSubdomains();
            assertEquals(1, subdomains.size());
            assertEquals("Sub-domain #0 is missing", TopologyServiceClientMock.SUBDOMAIN_0_NAME, subdomains.get(0)
                    .getName());
            final List<Container> containers = subdomains.get(0).getContainers();
            assertEquals(1, containers.size());
            assertEquals("Container #01 is missing", TopologyServiceClientMock.CONTAINER_01_NAME, containers.get(0)
                    .getContainerName());
        }

    }

}
